@@ -42,7 +42,7 @@ class BackendWorker;
 /**
  * @brief Thread safe logger.
  *
- * Logger must be obtained from create_or_get_logger(), therefore constructors are private
+ * Logger must be obtained from create_or_get_logger(), therefore, constructors are private
  */
 template <typename TFrontendOptions>
 class LoggerImpl : public detail::LoggerBase
@@ -66,118 +66,43 @@ public:
    * fundamental types.
    * This is the fastest way possible to log
    * @note This function is thread-safe.
-   * @param dynamic_log_level dynamic log level
    * @param macro_metadata metadata of the log message
    * @param fmt_args arguments
    *
    * @return true if the message is written to the queue, false if it is dropped (when a dropping queue is used)
    */
-  template <bool immediate_flush, bool has_dynamic_log_level, typename... Args>
-  QUILL_ATTRIBUTE_HOT bool log_statement(QUILL_MAYBE_UNUSED LogLevel dynamic_log_level,
-                                         MacroMetadata const* macro_metadata, Args&&... fmt_args)
+  template <bool enable_immediate_flush, typename... Args>
+  QUILL_ATTRIBUTE_HOT bool log_statement(MacroMetadata const* macro_metadata, Args&&... fmt_args)
   {
 #ifndef NDEBUG
-    if (has_dynamic_log_level)
-    {
-      assert((dynamic_log_level != LogLevel::None) &&
-             "When has_dynamic_log_level is set to true then dynamic_log_level should not be None");
-    }
+    assert((macro_metadata->event() != MacroMetadata::Event::LogWithRuntimeMetadataDeepCopy &&
+            macro_metadata->event() != MacroMetadata::Event::LogWithRuntimeMetadataShallowCopy &&
+            macro_metadata->event() != MacroMetadata::Event::LogWithRuntimeMetadataHybridCopy) &&
+           "Should not be called with MacroMetadata::Event::LogWithRuntimeMetadata");
 
-    if (dynamic_log_level != LogLevel::None)
-    {
-      assert((macro_metadata->log_level() == LogLevel::Dynamic) &&
-             "MacroMetadata LogLevel must be Dynamic when using a dynamic_log_level");
-
-      assert(has_dynamic_log_level &&
-             "When dynamic_log_level is used then has_dynamic_log_level must also be true");
-    }
-
-    if (macro_metadata->log_level() != LogLevel::Dynamic)
-    {
-      assert((dynamic_log_level == LogLevel::None) &&
-             "No dynamic_log_level should be set when MacroMetadata LogLevel is not Dynamic");
-    }
-
-    assert(valid.load(std::memory_order_acquire) && "Invalidated loggers can not log");
+    assert(_valid.load(std::memory_order_acquire) && "Invalidated loggers can not log");
 #endif
 
-    // Store the timestamp of the log statement at the start of the call. This gives more accurate
-    // timestamp especially if the queue is full
-    // This is very rare but might lead to out of order timestamp in the log file if we block on push for too long
+    // Store the timestamp of the log statement at the start of the call. This gives a more accurate
+    // timestamp, especially if the queue is full
+    uint64_t const current_timestamp = _get_message_timestamp();
 
-    uint64_t current_timestamp;
-
-    if (clock_source == ClockSourceType::Tsc)
-    {
-      current_timestamp = detail::rdtsc();
-    }
-    else if (clock_source == ClockSourceType::System)
-    {
-      current_timestamp = detail::get_timestamp_ns<std::chrono::system_clock>();
-    }
-    else if (user_clock)
-    {
-      current_timestamp = user_clock->now();
-    }
-    else
-    {
-      // not expected
-      current_timestamp = 0;
-    }
-
-    if (QUILL_UNLIKELY(thread_context == nullptr))
+    if (QUILL_UNLIKELY(_thread_context == nullptr))
     {
       // This caches the ThreadContext pointer to avoid repeatedly calling get_local_thread_context()
-      thread_context = detail::get_local_thread_context<frontend_options_t>();
+      _thread_context = detail::get_local_thread_context<frontend_options_t>();
     }
 
     // Need to know how much size we need from the queue
     size_t total_size = sizeof(current_timestamp) + (sizeof(uintptr_t) * 3) +
       detail::compute_encoded_size_and_cache_string_lengths(
-                          thread_context->get_conditional_arg_size_cache(), fmt_args...);
+                          _thread_context->get_conditional_arg_size_cache(), fmt_args...);
 
-    if constexpr (has_dynamic_log_level)
+    std::byte* write_buffer = _reserve_queue_space(total_size, macro_metadata);
+
+    if (QUILL_UNLIKELY(write_buffer == nullptr))
     {
-      // For the dynamic log level we want to add to the total size to store the dynamic log level
-      total_size += sizeof(dynamic_log_level);
-    }
-
-    std::byte* write_buffer = _prepare_write_buffer(total_size);
-
-    if constexpr ((frontend_options_t::queue_type == QueueType::BoundedDropping) ||
-                  (frontend_options_t::queue_type == QueueType::UnboundedDropping))
-    {
-      if (QUILL_UNLIKELY(write_buffer == nullptr))
-      {
-        // not enough space to push to queue message is dropped
-        if (macro_metadata->event() == MacroMetadata::Event::Log)
-        {
-          thread_context->increment_failure_counter();
-        }
-        return false;
-      }
-    }
-    else if constexpr ((frontend_options_t::queue_type == QueueType::BoundedBlocking) ||
-                       (frontend_options_t::queue_type == QueueType::UnboundedBlocking))
-    {
-      if (QUILL_UNLIKELY(write_buffer == nullptr))
-      {
-        if (macro_metadata->event() == MacroMetadata::Event::Log)
-        {
-          thread_context->increment_failure_counter();
-        }
-
-        do
-        {
-          if constexpr (frontend_options_t::blocking_queue_retry_interval_ns > 0)
-          {
-            std::this_thread::sleep_for(std::chrono::nanoseconds{frontend_options_t::blocking_queue_retry_interval_ns});
-          }
-
-          // not enough space to push to queue, keep trying
-          write_buffer = _prepare_write_buffer(total_size);
-        } while (write_buffer == nullptr);
-      }
+      return false;
     }
 
     // we have enough space in this buffer, and we will write to the buffer
@@ -192,14 +117,126 @@ public:
                                   detail::decode_and_store_args<detail::remove_cvref_t<Args>...>);
 
     // encode remaining arguments
-    detail::encode(write_buffer, thread_context->get_conditional_arg_size_cache(), fmt_args...);
+    detail::encode(write_buffer, _thread_context->get_conditional_arg_size_cache(), fmt_args...);
 
-    if constexpr (has_dynamic_log_level)
+#ifndef NDEBUG
+    assert((write_buffer > write_begin) && "write_buffer must be greater than write_begin");
+    assert(total_size == (static_cast<size_t>(write_buffer - write_begin)) &&
+           "The committed write bytes must be equal to the total_size requested bytes");
+#endif
+
+    _commit_log_statement<enable_immediate_flush>(total_size);
+
+    return true;
+  }
+
+  /**
+   * @brief Push a log message with runtime metadata to the spsc queue to be logged by the backend thread.
+   *
+   * Similar to log_statement but allows passing metadata that is only available at runtime.
+   *
+   * @note This function is thread-safe.
+   * @param macro_metadata Metadata of the log message
+   * @param fmt Format string for the log message
+   * @param file_path Source file path where the log statement was called
+   * @param function_name Function name where the log statement was called
+   * @param tags Optional tags associated with the log message
+   * @param line_number Line number in the source file
+   * @param log_level Severity level of the log message
+   * @param fmt_args Format arguments for the log message
+   *
+   * @return true if the message is written to the queue, false if it is dropped (when a dropping queue is used)
+   */
+  template <bool enable_immediate_flush, typename... Args>
+  QUILL_ATTRIBUTE_HOT bool log_statement_runtime_metadata(MacroMetadata const* macro_metadata,
+                                                          char const* fmt, char const* file_path,
+                                                          char const* function_name,
+                                                          char const* tags, uint32_t line_number,
+                                                          LogLevel log_level, Args&&... fmt_args)
+  {
+#ifndef NDEBUG
+    assert((macro_metadata->event() == MacroMetadata::Event::LogWithRuntimeMetadataDeepCopy ||
+            macro_metadata->event() == MacroMetadata::Event::LogWithRuntimeMetadataHybridCopy ||
+            macro_metadata->event() == MacroMetadata::Event::LogWithRuntimeMetadataShallowCopy) &&
+           "Should only be called with MacroMetadata::Event::LogWithRuntimeMetadata");
+
+    assert(_valid.load(std::memory_order_acquire) && "Invalidated loggers can not log");
+#endif
+
+    // Store the timestamp of the log statement at the start of the call. This gives a more accurate
+    // timestamp, especially if the queue is full
+    uint64_t const current_timestamp = _get_message_timestamp();
+
+    if (QUILL_UNLIKELY(_thread_context == nullptr))
     {
-      // write the dynamic log level
-      // The reason we write it last is that is less likely to break the alignment in the buffer
-      std::memcpy(write_buffer, &dynamic_log_level, sizeof(dynamic_log_level));
-      write_buffer += sizeof(dynamic_log_level);
+      // This caches the ThreadContext pointer to avoid repeatedly calling get_local_thread_context()
+      _thread_context = detail::get_local_thread_context<frontend_options_t>();
+    }
+
+    // Need to know how much size we need from the queue
+    // Here we need extra size for the metadata
+    size_t total_size{sizeof(current_timestamp) + (sizeof(uintptr_t) * 3)};
+
+    if (macro_metadata->event() == MacroMetadata::Event::LogWithRuntimeMetadataDeepCopy)
+    {
+      total_size += detail::compute_encoded_size_and_cache_string_lengths(
+        _thread_context->get_conditional_arg_size_cache(), fmt, file_path, function_name, tags,
+        line_number, log_level, fmt_args...);
+    }
+    else if (macro_metadata->event() == MacroMetadata::Event::LogWithRuntimeMetadataShallowCopy)
+    {
+      total_size += detail::compute_encoded_size_and_cache_string_lengths(
+        _thread_context->get_conditional_arg_size_cache(), static_cast<void const*>(fmt),
+        static_cast<void const*>(file_path), static_cast<void const*>(function_name),
+        static_cast<void const*>(tags), line_number, log_level, fmt_args...);
+    }
+    else if (macro_metadata->event() == MacroMetadata::Event::LogWithRuntimeMetadataHybridCopy)
+    {
+      total_size += detail::compute_encoded_size_and_cache_string_lengths(
+        _thread_context->get_conditional_arg_size_cache(), fmt, static_cast<void const*>(file_path),
+        static_cast<void const*>(function_name), tags, line_number, log_level, fmt_args...);
+    }
+    else
+    {
+      return false;
+    }
+
+    std::byte* write_buffer = _reserve_queue_space(total_size, macro_metadata);
+
+    if (QUILL_UNLIKELY(write_buffer == nullptr))
+    {
+      return false;
+    }
+
+    // we have enough space in this buffer, and we will write to the buffer
+
+#ifndef NDEBUG
+    std::byte const* const write_begin = write_buffer;
+    assert(write_begin);
+#endif
+
+    // first encode a header
+    write_buffer = _encode_header(write_buffer, current_timestamp, macro_metadata, this,
+                                  detail::decode_and_store_args<detail::remove_cvref_t<Args>...>);
+
+    if (macro_metadata->event() == MacroMetadata::Event::LogWithRuntimeMetadataDeepCopy)
+    {
+      // encode runtime metadata and remaining arguments
+      detail::encode(write_buffer, _thread_context->get_conditional_arg_size_cache(), fmt,
+                     file_path, function_name, tags, line_number, log_level, fmt_args...);
+    }
+    else if (macro_metadata->event() == MacroMetadata::Event::LogWithRuntimeMetadataShallowCopy)
+    {
+      detail::encode(write_buffer, _thread_context->get_conditional_arg_size_cache(),
+                     static_cast<void const*>(fmt), static_cast<void const*>(file_path),
+                     static_cast<void const*>(function_name), static_cast<void const*>(tags),
+                     line_number, log_level, fmt_args...);
+    }
+    else if (macro_metadata->event() == MacroMetadata::Event::LogWithRuntimeMetadataHybridCopy)
+    {
+      detail::encode(write_buffer, _thread_context->get_conditional_arg_size_cache(), fmt,
+                     static_cast<void const*>(file_path), static_cast<void const*>(function_name),
+                     tags, line_number, log_level, fmt_args...);
     }
 
 #ifndef NDEBUG
@@ -208,21 +245,16 @@ public:
            "The committed write bytes must be equal to the total_size requested bytes");
 #endif
 
-    thread_context->get_spsc_queue<frontend_options_t::queue_type>().finish_and_commit_write(total_size);
-
-    if constexpr (immediate_flush)
-    {
-      this->flush_log();
-    }
+    _commit_log_statement<enable_immediate_flush>(total_size);
 
     return true;
   }
 
   /**
    * Init a backtrace for this logger.
-   * Stores messages logged with LOG_BACKTRACE in a ring buffer messages and displays them later on demand.
+   * Stores messages logged with LOG_BACKTRACE in a ring buffer and displays them later on demand.
    * @param max_capacity The max number of messages to store in the backtrace
-   * @param flush_level If this loggers logs any message higher or equal to this severity level the backtrace will also get flushed.
+   * @param flush_level If this logger logs any message higher or equal to this severity level, the backtrace will also get flushed.
    * Default level is None meaning the user has to call flush_backtrace explicitly
    */
   void init_backtrace(uint32_t max_capacity, LogLevel flush_level = LogLevel::None)
@@ -233,13 +265,13 @@ public:
 
     // we pass this message to the queue and also pass capacity as arg
     // We do not want to drop the message if a dropping queue is used
-    while (!this->log_statement<false, false>(LogLevel::None, &macro_metadata, max_capacity))
+    while (!this->template log_statement<false>(&macro_metadata, max_capacity))
     {
       std::this_thread::sleep_for(std::chrono::nanoseconds{100});
     }
 
     // Also store the desired flush log level
-    backtrace_flush_level.store(flush_level, std::memory_order_relaxed);
+    _backtrace_flush_level.store(flush_level, std::memory_order_relaxed);
   }
 
   /**
@@ -252,7 +284,7 @@ public:
       "", "", "", nullptr, LogLevel::Critical, MacroMetadata::Event::FlushBacktrace};
 
     // We do not want to drop the message if a dropping queue is used
-    while (!this->log_statement<false, false>(LogLevel::None, &macro_metadata))
+    while (!this->template log_statement<false>(&macro_metadata))
     {
       std::this_thread::sleep_for(std::chrono::nanoseconds{100});
     }
@@ -284,8 +316,7 @@ public:
     std::atomic<bool>* backend_thread_flushed_ptr = &backend_thread_flushed;
 
     // We do not want to drop the message if a dropping queue is used
-    while (!this->log_statement<false, false>(
-      LogLevel::None, &macro_metadata, reinterpret_cast<uintptr_t>(backend_thread_flushed_ptr)))
+    while (!this->log_statement<false>(&macro_metadata, reinterpret_cast<uintptr_t>(backend_thread_flushed_ptr)))
     {
       if (sleep_duration_ns > 0)
       {
@@ -324,10 +355,113 @@ private:
         static_cast<PatternFormatterOptions&&>(pattern_formatter_options), clock_source, user_clock)
 
   {
-    if (this->user_clock)
+    if (this->_user_clock)
     {
       // if a user clock is provided then set the ClockSourceType to User
-      this->clock_source = ClockSourceType::User;
+      this->_clock_source = ClockSourceType::User;
+    }
+  }
+
+  /**
+   * @brief Get the current timestamp for a log message based on the configured clock source
+   * @return uint64_t The current timestamp in nanoseconds
+   */
+  QUILL_NODISCARD QUILL_ATTRIBUTE_HOT uint64_t _get_message_timestamp() const
+  {
+    // This is very rare but might lead to out-of-order timestamp in the log file if a thread blocks
+    // on _reserve_queue_space for too long
+
+    if (_clock_source == ClockSourceType::Tsc)
+    {
+      return detail::rdtsc();
+    }
+
+    if (_clock_source == ClockSourceType::System)
+    {
+      return detail::get_timestamp_ns<std::chrono::system_clock>();
+    }
+
+    if (_user_clock)
+    {
+      return _user_clock->now();
+    }
+
+    // not expected
+    return 0;
+  }
+
+  /**
+   * Reserve space in the thread's SPSC queue for a log message
+   * @param total_size The total size in bytes needed for the log message
+   * @param macro_metadata Metadata of the log message, used to increment failure counter if needed
+   * @return std::byte* Pointer to the reserved space in the queue, or nullptr if space cannot be allocated
+   */
+  QUILL_NODISCARD QUILL_ATTRIBUTE_HOT std::byte* _reserve_queue_space(size_t total_size,
+                                                                      MacroMetadata const* macro_metadata)
+  {
+    std::byte* write_buffer =
+      _thread_context->get_spsc_queue<frontend_options_t::queue_type>().prepare_write(total_size);
+
+    if constexpr ((frontend_options_t::queue_type == QueueType::BoundedDropping) ||
+                  (frontend_options_t::queue_type == QueueType::UnboundedDropping))
+    {
+      if (QUILL_UNLIKELY(write_buffer == nullptr))
+      {
+        // not enough space to push to queue message is dropped
+        if (macro_metadata->event() == MacroMetadata::Event::Log)
+        {
+          _thread_context->increment_failure_counter();
+        }
+      }
+    }
+    else if constexpr ((frontend_options_t::queue_type == QueueType::BoundedBlocking) ||
+                       (frontend_options_t::queue_type == QueueType::UnboundedBlocking))
+    {
+      if (QUILL_UNLIKELY(write_buffer == nullptr))
+      {
+        if (macro_metadata->event() == MacroMetadata::Event::Log)
+        {
+          _thread_context->increment_failure_counter();
+        }
+
+        do
+        {
+          if constexpr (frontend_options_t::blocking_queue_retry_interval_ns > 0)
+          {
+            std::this_thread::sleep_for(std::chrono::nanoseconds{frontend_options_t::blocking_queue_retry_interval_ns});
+          }
+
+          // not enough space to push to queue, keep trying
+          write_buffer =
+            _thread_context->get_spsc_queue<frontend_options_t::queue_type>().prepare_write(total_size);
+        } while (write_buffer == nullptr);
+      }
+    }
+
+    return write_buffer;
+  }
+
+  /**
+   * Commit a log statement to the queue and optionally flush it
+   * @param total_size The total size in bytes of the committed log message
+   */
+  template <bool enable_immediate_flush>
+  QUILL_ATTRIBUTE_HOT void _commit_log_statement(size_t total_size)
+  {
+    _thread_context->get_spsc_queue<frontend_options_t::queue_type>().finish_and_commit_write(total_size);
+
+    if constexpr (enable_immediate_flush)
+    {
+      uint32_t const threshold = _message_flush_threshold.load(std::memory_order_relaxed);
+      if (QUILL_UNLIKELY(threshold != 0))
+      {
+        uint32_t const prev = _messages_since_last_flush.fetch_add(1, std::memory_order_relaxed);
+        if ((prev + 1) >= threshold)
+        {
+          _messages_since_last_flush.store(0, std::memory_order_relaxed);
+          this->flush_log();
+        }
+      }
     }
   }
 
@@ -358,14 +492,6 @@ private:
 
     return write_buffer;
   }
-
-  /**
-   * Prepares a write buffer for the given context and size.
-   */
-  QUILL_NODISCARD QUILL_ATTRIBUTE_HOT std::byte* _prepare_write_buffer(size_t total_size)
-  {
-    return thread_context->get_spsc_queue<frontend_options_t::queue_type>().prepare_write(total_size);
-  }
 };
 
 using Logger = LoggerImpl<FrontendOptions>;
