@@ -24,14 +24,19 @@ class MacroMetadata
 public:
   enum Event : uint8_t
   {
+    None,
     Log,
     InitBacktrace,
     FlushBacktrace,
     Flush,
-    LogWithRuntimeMetadata,
+    LogWithRuntimeMetadataDeepCopy,
+    LogWithRuntimeMetadataHybridCopy,
+    LogWithRuntimeMetadataShallowCopy,
     LoggerRemovalRequest
   };
 
+  constexpr MacroMetadata() = default;
+
   constexpr MacroMetadata(char const* source_location, char const* caller_function,
                           char const* message_format, char const* tags, LogLevel log_level, Event event) noexcept
     : _source_location(source_location),
@@ -43,7 +48,6 @@ public:
       _log_level(log_level),
       _event(event)
   {
-    _set_named_args_flag(_contains_named_args(message_format));
   }
 
   QUILL_NODISCARD char const* source_location() const noexcept { return _source_location; }
@@ -77,7 +81,10 @@ public:
 
   QUILL_NODISCARD char const* tags() const noexcept { return _tags; }
 
-  QUILL_NODISCARD bool has_named_args() const noexcept { return _format_flags & NAMED_ARGS_FLAG; }
+  QUILL_NODISCARD constexpr bool has_named_args() const noexcept
+  {
+    return _contains_named_args(_message_format);
+  }
 
   QUILL_NODISCARD Event event() const noexcept { return _event; }
 
@@ -155,7 +162,7 @@ private:
     while (*source_location)
     {
       char cur = *source_location++;
-      if (cur == '/' || cur == PATH_PREFERRED_SEPARATOR)
+      if (cur == '/' || cur == detail::PATH_PREFERRED_SEPARATOR)
       {
         file = source_location;
       }
@@ -171,38 +178,17 @@ private:
     return static_cast<uint16_t>(separator_index);
   }
 
-  /***/
-  constexpr void _set_named_args_flag(bool value) noexcept
-  {
-    if (value)
-    {
-      _format_flags |= NAMED_ARGS_FLAG;
-    }
-    else
-    {
-      _format_flags &= static_cast<uint8_t>(~NAMED_ARGS_FLAG);
-    }
-  }
-
 private:
-  // define our own PATH_PREFERRED_SEPARATOR to not include <filesystem>
-#if defined(_WIN32) && !defined(__CYGWIN__)
-  static constexpr wchar_t PATH_PREFERRED_SEPARATOR = L'\\';
-#else
-  static constexpr char PATH_PREFERRED_SEPARATOR = '/';
-#endif
-
   static constexpr uint8_t NAMED_ARGS_FLAG = 0x01;
 
-  char const* _source_location;
-  char const* _caller_function;
-  char const* _message_format;
-  char const* _tags;
-  uint16_t _colon_separator_pos;
-  uint16_t _file_name_pos;
-  LogLevel _log_level;
-  Event _event;
-  uint8_t _format_flags{0};
+  char const* _source_location{nullptr};
+  char const* _caller_function{nullptr};
+  char const* _message_format{nullptr};
+  char const* _tags{nullptr};
+  uint16_t _colon_separator_pos{0};
+  uint16_t _file_name_pos{0};
+  LogLevel _log_level{LogLevel::None};
+  Event _event{Event::None};
 };
 
 static_assert(sizeof(MacroMetadata) <= detail::QUILL_CACHE_LINE_SIZE,
