@@ -244,7 +244,7 @@ private:
 /**
  * @brief The RotatingSink class
  */
-template<typename TBase>
+template <typename TBase>
 class RotatingSink : public TBase
 {
 public:
@@ -261,8 +261,8 @@ public:
    * @param start_time start time
    */
   RotatingSink(fs::path const& filename, RotatingFileSinkConfig const& config,
-                   FileEventNotifier file_event_notifier = FileEventNotifier{},
-                   std::chrono::system_clock::time_point start_time = std::chrono::system_clock::now())
+               FileEventNotifier file_event_notifier = FileEventNotifier{},
+               std::chrono::system_clock::time_point start_time = std::chrono::system_clock::now())
     : base_type(filename, static_cast<FileSinkConfig const&>(config), std::move(file_event_notifier), false),
       _config(config)
   {
@@ -321,8 +321,8 @@ public:
     if (this->is_null())
     {
       base_type::write_log(log_metadata, log_timestamp, thread_id, thread_name, process_id,
-                            logger_name, log_level, log_level_description, log_level_short_code,
-                            named_args, log_message, log_statement);
+                           logger_name, log_level, log_level_description, log_level_short_code,
+                           named_args, log_message, log_statement);
       return;
     }
 
@@ -342,8 +342,8 @@ public:
 
     // write to file
     base_type::write_log(log_metadata, log_timestamp, thread_id, thread_name, process_id,
-                          logger_name, log_level, log_level_description, log_level_short_code,
-                          named_args, log_message, log_statement);
+                         logger_name, log_level, log_level_description, log_level_short_code,
+                         named_args, log_message, log_statement);
   }
 
 private:
@@ -478,7 +478,7 @@ private:
     // if we are starting in "w" mode, then we also should clean all previous log files of the previous run
     if (_config.remove_old_files() && (open_mode == "w"))
     {
-      for (const auto& entry : fs::directory_iterator(fs::current_path() / filename.parent_path()))
+      for (auto const& entry : fs::directory_iterator(fs::current_path() / filename.parent_path()))
       {
         if (entry.path().extension().string() != filename.extension().string())
         {
@@ -540,7 +540,7 @@ private:
     else if (open_mode == "a")
     {
       // we need to recover the index from the existing files
-      for (const auto& entry : fs::directory_iterator(fs::current_path() / filename.parent_path()))
+      for (auto const& entry : fs::directory_iterator(fs::current_path() / filename.parent_path()))
       {
         // is_directory() does not exist in std::experimental::filesystem
         if (entry.path().extension().string() != filename.extension().string())
